==> opennms-services-1.7.6-sources.jar: org/opennms/netmgt/poller/monitors/MemcachedMonitor.java <==
//
// This file is part of the OpenNMS(R) Application.
//
// OpenNMS(R) is Copyright (C) 2002-2009 The OpenNMS Group, Inc. All rights reserved.
// OpenNMS(R) is a derivative work, containing both original code, included code and modified
// code that was published under the GNU General Public License. Copyrights for modified
// and included code are below.
//
// OpenNMS(R) is a registered trademark of The OpenNMS Group, Inc.
//
// Original code base Copyright (C) 1999-2001 Oculan Corp. All rights reserved.
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
//
// For more information contact:
//      OpenNMS Licensing <license@opennms.org>
//      http://www.opennms.org/
//      http://www.opennms.com/
//
// Tab Size = 8
//

package org.opennms.netmgt.poller.monitors;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.InterruptedIOException;
import java.io.OutputStreamWriter;
import java.net.ConnectException;
import java.net.InetAddress;
import java.net.InetSocketAddress;
import java.net.NoRouteToHostException;
import java.net.Socket;
import java.util.LinkedHashMap;
import java.util.Map;

import org.apache.log4j.Level;
import org.opennms.core.utils.ParameterMap;
import org.opennms.core.utils.TimeoutTracker;
import org.opennms.netmgt.model.PollStatus;
import org.opennms.netmgt.poller.Distributable;
import org.opennms.netmgt.poller.MonitoredService;

/**
 * This class is designed to be used by the service poller framework to test the
 * availability of the Memcached service on remote interfaces. The class implements
 * the ServiceMonitor interface that allows it to be used along with other
 * plug-ins by the service poller framework.
 * 
 * @author <A HREF="mailto:ranger@opennms.org">Benjamin Reed</A>
 * @author <A HREF="http://www.opennms.org/">OpenNMS</A>
 * 
 * 
 */
@Distributable
final public class MemcachedMonitor extends IPv4Monitor {

    /**
     * Default FTP port.
     */
    private static final int DEFAULT_PORT = 11211;

    /**
     * Default retries.
     */
    private static final int DEFAULT_RETRY = 0;

    /**
     * Default timeout. Specifies how long (in milliseconds) to block waiting
     * for data from the monitored interface.
     */
    private static final int DEFAULT_TIMEOUT = 3000; // 3 second timeout on read()

    private static final String[] m_keys = new String[] {
        "uptime", "rusageuser", "rusage_system",
        "curr_items", "totalitems", "bytes",
        "currconnections", "totalconnections", "connectionstructure",
        "cmdget", "cmdset", "gethits", "getmisses", "evictions",
        "bytesread", "byteswritten", "threads"
    };
    
    /**
     * Poll the specified address for Memcached service availability.
     * 
     * @param parameters
     *            The package parameters (timeout, retry, etc...) to be used for this poll.
     * @param iface
     *            The network interface to test the service on.
     * @return The availability of the interface and if a transition event should be suppressed.
     * 
     */
    public PollStatus poll(MonitoredService svc, Map<String, Object> parameters) {

        TimeoutTracker timeoutTracker = new TimeoutTracker(parameters, DEFAULT_RETRY, DEFAULT_TIMEOUT);
        
        int port = ParameterMap.getKeyedInteger(parameters, "port", DEFAULT_PORT);

        // Extract the address
        InetAddress ipv4Addr = (InetAddress) svc.getAddress();
        String host = ipv4Addr.getHostAddress();

        if (log().isDebugEnabled())
            log().debug("polling interface: " + host + timeoutTracker);

        PollStatus serviceStatus = PollStatus.unavailable();

        for(timeoutTracker.reset(); timeoutTracker.shouldRetry() && !serviceStatus.isAvailable(); timeoutTracker.nextAttempt()) {
            Socket socket = null;
            try {

                timeoutTracker.startAttempt();
                
                socket = new Socket();
                socket.connect(new InetSocketAddress(ipv4Addr, port), timeoutTracker.getConnectionTimeout());
                socket.setSoTimeout(timeoutTracker.getSoTimeout());
                log().debug("connected to host: " + host + " on port: " + port);

                // We're connected, so upgrade status to unresponsive
                serviceStatus = PollStatus.unresponsive();

                OutputStreamWriter osw = new OutputStreamWriter(socket.getOutputStream());
                osw.write("stats\n");
                osw.flush();

                // Allocate a line reader
                BufferedReader reader = new BufferedReader(new InputStreamReader(socket.getInputStream()));

                Map<String, Number> statProps = new LinkedHashMap<String,Number>();
                for (String key : m_keys) {
                    statProps.put(key, null);
                }

                String line = null;
                if (reader != null) {
                    do {
                        line = reader.readLine();
                        String[] statEntry = line.trim().split("\\s", 3);
                        if (statEntry[0].equals("STAT")) {
                            try {
                                Number value;
                                if (statEntry[2].contains(".")) {
                                    value = Double.parseDouble(statEntry[2]);
                                } else {
                                    value = Long.parseLong(statEntry[2]);
                                }
                                String key = statEntry[1].toLowerCase();
                                key = key.replaceAll("_", "");
                                if (key.length() > 19) {
                                    key = key.substring(0, 19);
                                }
                                if (statProps.containsKey(key)) {
                                    statProps.put(key, value);
                                }
                            } catch (Exception e) {
                                // ignore errors parsing
                            }
                        } else if (statEntry[0].equals("END")) {
                            serviceStatus = PollStatus.available();
                            osw.write("quit\n");
                            osw.flush();
                            break;
                        }
                    } while (line != null);
                } else {
                    log().warn("unable to read after sending stats request");
                }

                serviceStatus.setProperties(statProps);
                serviceStatus.setResponseTime(timeoutTracker.elapsedTimeInMillis());
            } catch (ConnectException e) {
                // Connection refused!! Continue to retry.
            	serviceStatus = logDown(Level.DEBUG, "Connection refused by host "+host, e);
            } catch (NoRouteToHostException e) {
            	// No route to host!! Try retries anyway in case strict timeouts are enabled
                serviceStatus = logDown(Level.INFO, "Unable to test host " + host + ", no route available", e);
            } catch (InterruptedIOException e) {
            	serviceStatus = logDown(Level.DEBUG, "did not connect to host " + host +" within timeout: " + timeoutTracker);
            } catch (IOException e) {
            	serviceStatus = logDown(Level.INFO, "Error communicating with host " + host, e);
            } catch (Throwable t) {
                serviceStatus = logDown(Level.WARN, "Undeclared throwable exception caught contacting host " + host, t);
            } finally {
                try {
                    if (socket != null) {
                        socket.close();
                        socket = null;
                    }
                } catch (IOException e) {
                }
            }
            
        }

        //
        // return the status of the service
        //
        return serviceStatus;
    }

}

==> opennms-services-1.7.91-sources.jar: org/opennms/netmgt/poller/monitors/MemcachedMonitor.java <==
//
// This file is part of the OpenNMS(R) Application.
//
// OpenNMS(R) is Copyright (C) 2002-2009 The OpenNMS Group, Inc. All rights reserved.
// OpenNMS(R) is a derivative work, containing both original code, included code and modified
// code that was published under the GNU General Public License. Copyrights for modified
// and included code are below.
//
// OpenNMS(R) is a registered trademark of The OpenNMS Group, Inc.
//
// Original code base Copyright (C) 1999-2001 Oculan Corp. All rights reserved.
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
//
// For more information contact:
//      OpenNMS Licensing <license@opennms.org>
//      http://www.opennms.org/
//      http://www.opennms.com/
//
// Tab Size = 8
//

package org.opennms.netmgt.poller.monitors;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.InterruptedIOException;
import java.io.OutputStreamWriter;
import java.net.ConnectException;
import java.net.InetAddress;
import java.net.InetSocketAddress;
import java.net.NoRouteToHostException;
import java.net.Socket;
import java.util.LinkedHashMap;
import java.util.Map;

import org.apache.log4j.Level;
import org.opennms.core.utils.ParameterMap;
import org.opennms.core.utils.TimeoutTracker;
import org.opennms.netmgt.model.PollStatus;
import org.opennms.netmgt.poller.Distributable;
import org.opennms.netmgt.poller.MonitoredService;

/**
 * This class is designed to be used by the service poller framework to test the
 * availability of the Memcached service on remote interfaces. The class implements
 * the ServiceMonitor interface that allows it to be used along with other
 * plug-ins by the service poller framework.
 * 
 * @author <A HREF="mailto:ranger@opennms.org">Benjamin Reed</A>
 * @author <A HREF="http://www.opennms.org/">OpenNMS</A>
 * 
 * 
 */
@Distributable
final public class MemcachedMonitor extends IPv4Monitor {

    /**
     * Default FTP port.
     */
    private static final int DEFAULT_PORT = 11211;

    /**
     * Default retries.
     */
    private static final int DEFAULT_RETRY = 0;

    /**
     * Default timeout. Specifies how long (in milliseconds) to block waiting
     * for data from the monitored interface.
     */
    private static final int DEFAULT_TIMEOUT = 3000; // 3 second timeout on read()

    private static final String[] m_keys = new String[] {
        "uptime", "rusageuser", "rusage_system",
        "curr_items", "totalitems", "bytes",
        "currconnections", "totalconnections", "connectionstructure",
        "cmdget", "cmdset", "gethits", "getmisses", "evictions",
        "bytesread", "byteswritten", "threads"
    };
    
    /**
     * Poll the specified address for Memcached service availability.
     * 
     * @param parameters
     *            The package parameters (timeout, retry, etc...) to be used for this poll.
     * @param iface
     *            The network interface to test the service on.
     * @return The availability of the interface and if a transition event should be suppressed.
     * 
     */
    public PollStatus poll(MonitoredService svc, Map<String, Object> parameters) {

        TimeoutTracker timeoutTracker = new TimeoutTracker(parameters, DEFAULT_RETRY, DEFAULT_TIMEOUT);
        
        int port = ParameterMap.getKeyedInteger(parameters, "port", DEFAULT_PORT);

        // Extract the address
        InetAddress ipv4Addr = svc.getAddress();
        String host = ipv4Addr.getHostAddress();

        if (log().isDebugEnabled())
            log().debug("polling interface: " + host + timeoutTracker);

        PollStatus serviceStatus = PollStatus.unavailable();

        for(timeoutTracker.reset(); timeoutTracker.shouldRetry() && !serviceStatus.isAvailable(); timeoutTracker.nextAttempt()) {
            Socket socket = null;
            try {

                timeoutTracker.startAttempt();
                
                socket = new Socket();
                socket.connect(new InetSocketAddress(ipv4Addr, port), timeoutTracker.getConnectionTimeout());
                socket.setSoTimeout(timeoutTracker.getSoTimeout());
                log().debug("connected to host: " + host + " on port: " + port);

                // We're connected, so upgrade status to unresponsive
                serviceStatus = PollStatus.unresponsive();

                OutputStreamWriter osw = new OutputStreamWriter(socket.getOutputStream(), "UTF-8");
                osw.write("stats\n");
                osw.flush();

                // Allocate a line reader
                BufferedReader reader = new BufferedReader(new InputStreamReader(socket.getInputStream()));

                Map<String, Number> statProps = new LinkedHashMap<String,Number>();
                for (String key : m_keys) {
                    statProps.put(key, null);
                }

                String line = null;
                if (reader != null) {
                    do {
                        line = reader.readLine();
                        String[] statEntry = line.trim().split("\\s", 3);
                        if (statEntry[0].equals("STAT")) {
                            try {
                                Number value;
                                if (statEntry[2].contains(".")) {
                                    value = Double.parseDouble(statEntry[2]);
                                } else {
                                    value = Long.parseLong(statEntry[2]);
                                }
                                String key = statEntry[1].toLowerCase();
                                key = key.replaceAll("_", "");
                                if (key.length() > 19) {
                                    key = key.substring(0, 19);
                                }
                                if (statProps.containsKey(key)) {
                                    statProps.put(key, value);
                                }
                            } catch (Exception e) {
                                // ignore errors parsing
                            }
                        } else if (statEntry[0].equals("END")) {
                            serviceStatus = PollStatus.available();
                            osw.write("quit\n");
                            osw.flush();
                            break;
                        }
                    } while (line != null);
                } else {
                    log().warn("unable to read after sending stats request");
                }

                serviceStatus.setProperties(statProps);
                serviceStatus.setResponseTime(timeoutTracker.elapsedTimeInMillis());
            } catch (ConnectException e) {
                // Connection refused!! Continue to retry.
            	serviceStatus = logDown(Level.DEBUG, "Connection refused by host "+host, e);
            } catch (NoRouteToHostException e) {
            	// No route to host!! Try retries anyway in case strict timeouts are enabled
                serviceStatus = logDown(Level.INFO, "Unable to test host " + host + ", no route available", e);
            } catch (InterruptedIOException e) {
            	serviceStatus = logDown(Level.DEBUG, "did not connect to host " + host +" within timeout: " + timeoutTracker);
            } catch (IOException e) {
            	serviceStatus = logDown(Level.INFO, "Error communicating with host " + host, e);
            } catch (Throwable t) {
                serviceStatus = logDown(Level.WARN, "Undeclared throwable exception caught contacting host " + host, t);
            } finally {
                try {
                    if (socket != null) {
                        socket.close();
                        socket = null;
                    }
                } catch (IOException e) {
                }
            }
            
        }

        //
        // return the status of the service
        //
        return serviceStatus;
    }

}
